/*
 * Copyright (c) 2000-2007 C.J. Schmitt
 * schmclug, 14.08.2007 13:49:39
 *
 * This program is free software;
 * you can redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software Foundation;
 * either version 3 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * See http://www.gnu.org/licenses.
 */


package net.illuimago.cjs.alphatab;

// import com.sun.java.swing.SwingUtilities2;
// import sun.swing.SwingUtilities2;

import javax.swing.plaf.basic.BasicTabbedPaneUI;
import javax.swing.plaf.ComponentUI;
import javax.swing.*;
import javax.swing.text.View;
import java.awt.*;
import java.lang.reflect.Method;
import java.lang.reflect.InvocationTargetException;

public class AlignedTabbedPaneUI extends BasicTabbedPaneUI {
    private final AlphaTab alphaTabPlugin;

    public AlignedTabbedPaneUI(final AlphaTab alphaTabPlugin) {
        this.textIconGap = 1;
        this.alphaTabPlugin = alphaTabPlugin;
    }

    public static ComponentUI createUI(JComponent c) {
        return new AlignedTabbedPaneUI(null);
    }

    protected void layoutLabel(int tabPlacement,
                               FontMetrics metrics, int tabIndex,
                               String title, Icon icon,
                               Rectangle tabRect, Rectangle iconRect,
                               Rectangle textRect, boolean isSelected) {
        textRect.x = textRect.y = iconRect.x = iconRect.y = 0;
        SwingUtilities.layoutCompoundLabel(
                tabPane,
                metrics, title, icon,
                SwingUtilities.CENTER,
                SwingUtilities.LEADING, //changed from CENTER to LEADING
                SwingUtilities.CENTER,
                SwingUtilities.TRAILING,
                tabRect,
                iconRect,
                textRect,
                1);

        int xNudge = getTabLabelShiftX(tabPlacement, tabIndex, isSelected);
        int yNudge = getTabLabelShiftY(tabPlacement, tabIndex, isSelected);
        iconRect.x += xNudge;
        iconRect.y += yNudge;
        textRect.x += xNudge;
        textRect.y += yNudge;
    }


    protected void paintText(Graphics graphics, int i, Font font, FontMetrics fontMetrics, int i1, String title,
                             Rectangle rt, boolean isSelected) {
        if (this.alphaTabPlugin != null && this.alphaTabPlugin.isHighlightTitle() && isSelected) {
            font = font.deriveFont(Font.BOLD);
            graphics.setFont(font);
        }
        if (this.alphaTabPlugin != null && this.alphaTabPlugin.isTruncateTitle() &&
                this.alphaTabPlugin.getNumOfCharsInTitleAsInt() > 0 &&
                title.length() > this.alphaTabPlugin.getNumOfCharsInTitleAsInt()) {
            title = title.substring(0, this.alphaTabPlugin.getNumOfCharsInTitleAsInt()) + "...";
        }
        super.paintText(graphics, i, font, fontMetrics, i1, title, rt, isSelected);
    }


    protected int calculateTabWidth(int tabPlacement, int tabIndex, FontMetrics metrics) {
        Icon icon = getIconForTab(tabIndex);
        Insets tabInsets = getTabInsets(tabPlacement, tabIndex);
        int width = tabInsets.left + tabInsets.right + 3;

        if (icon != null) {
            width += icon.getIconWidth() + textIconGap;
        }
        // plain text
        String title = tabPane.getTitleAt(tabIndex);
        if (this.alphaTabPlugin != null && this.alphaTabPlugin.isTruncateTitle() &&
                this.alphaTabPlugin.getNumOfCharsInTitleAsInt() > 0 &&
                title.length() > this.alphaTabPlugin.getNumOfCharsInTitleAsInt()) {
            title = title.substring(0, this.alphaTabPlugin.getNumOfCharsInTitleAsInt());
        }

        width += this.calcStringWidth(tabPane, metrics, title);

        if (this.alphaTabPlugin.isHighlightTitle())
            width = ((int) (width * 1.08));

        return width;
    }

    Method m = null;

    private int calcStringWidth(javax.swing.JComponent jComponent, java.awt.FontMetrics fontMetrics, java.lang.String functionJs) {
        Class clazz;
        if (m == null) {
            try {
                clazz = Class.forName("com.sun.java.swing.SwingUtilities2");
            }
            catch (ClassNotFoundException e) {
                try {
                    clazz = Class.forName("sun.swing.SwingUtilities2");
                }
                catch (ClassNotFoundException e1) {
                    e1.printStackTrace();
                    return functionJs.length() * CHAR_WIDTH;
                }
            }
            try {
                this.m = clazz.getMethod("stringWidth", JComponent.class, FontMetrics.class, String.class);
            }
            catch (NoSuchMethodException e) {
                e.printStackTrace();
                return functionJs.length() * CHAR_WIDTH;
            }
        }
        try {
            return ((Integer) this.m.invoke(null, jComponent, fontMetrics, functionJs)).intValue();
        }
        catch (Exception e) {
            e.printStackTrace();
            return functionJs.length() * CHAR_WIDTH;
        }
    }

    private final static int CHAR_WIDTH = 4;
}
