/*
 * Copyright (c) 2000-2007 C.J. Schmitt
 * schmclug, 14.08.2007 13:20:16
 *
 * This program is free software;
 * you can redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software Foundation;
 * either version 3 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * See http://www.gnu.org/licenses.
 */


package net.illuimago.cjs.alphatab;

import javax.swing.*;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;

public class AlphaTabConfigurationForm
{
    private JCheckBox highlightSelectedEditorTabCheckBox;
    private JPanel mainPanel;
    private JCheckBox truncateTitlesLongerThanCheckBox;
    private JTextField numberOfCharsTextField;
    private JCheckBox sortCaseSensitiveCheckBox;

    public void setData(AlphaTab data)
    {
        truncateTitlesLongerThanCheckBox.setSelected(data.isTruncateTitle());
        numberOfCharsTextField.setText(data.getNumOfCharsInTitle());
        highlightSelectedEditorTabCheckBox.setSelected(data.isHighlightTitle());
        this.numberOfCharsTextField.setEditable(data.isTruncateTitle());
        this.sortCaseSensitiveCheckBox.setSelected(data.isCaseSensitive());
    }

    public void getData(AlphaTab data)
    {
        data.setTruncateTitle(truncateTitlesLongerThanCheckBox.isSelected());
        data.setNumOfCharsInTitle(numberOfCharsTextField.getText());
        data.setHighlightTitle(highlightSelectedEditorTabCheckBox.isSelected());
        data.setCaseSensitive(this.sortCaseSensitiveCheckBox.isSelected());
    }

    public boolean isModified(AlphaTab data)
    {
        if (truncateTitlesLongerThanCheckBox.isSelected() != data.isTruncateTitle()) return true;
        if (numberOfCharsTextField.getText() != null ? !numberOfCharsTextField.getText().equals(data.getNumOfCharsInTitle()) : data.getNumOfCharsInTitle() != null)
            return true;
        if (highlightSelectedEditorTabCheckBox.isSelected() != data.isHighlightTitle()) return true;
        if (this.sortCaseSensitiveCheckBox.isSelected() != data.isCaseSensitive()) return true;
        return false;
    }

    // Method returns the root component of the form
    public JComponent getRootComponent()
    {
        this.truncateTitlesLongerThanCheckBox.addChangeListener(new ChangeListener()
        {
            public void stateChanged(final ChangeEvent e)
            {
                AlphaTabConfigurationForm.this.numberOfCharsTextField.setEditable(
                        AlphaTabConfigurationForm.this.truncateTitlesLongerThanCheckBox.isSelected());
            }
        });
        return mainPanel;
    }
}
